# -*- coding: utf-8 -*-
##------------------------------------------
##  FormOCR Program      Ver 0.01
##      with easyocr & cvui
##          platform: linux / windows
##
##               2022.05.10 Masahiro Izutsu
##------------------------------------------
## easyform.py       (tryocr_go.py  ver 0.03)
##                  (formocr.py    ver 0.01)

title = 'EasyOCR Program  Ver 0.01'

# Color Escape Code
GREEN = '\033[1;32m'
RED = '\033[1;31m'
NOCOLOR = '\033[0m'
YELLOW = '\033[1;33m'

# 定数定義
LINE_WORD_BOX_COLOR = (0, 0, 240)
WORD_BOX_COLOR = (255, 0, 0)
CONTENTS_COLOR = (0, 128, 0)
from os.path import expanduser
DEF_CSV_PATH = expanduser('easyform.csv')
CONFIG_FILE = expanduser('easyform.yaml')
MAPING_FILE = expanduser('easyform_templ.yaml')

# import処理
from PIL import Image
import sys
import easyocr
import cv2
import cvui
import argparse
import myfunction
import numpy as np
import mylib_gui
import mylib_frame
import mylib_preprocess
import mylib_screen
import platform
from tkinter import filedialog
import mylib_yaml
import mylib_text
import mylib_csv
import copy
import os
import tkinter
from tkinter import messagebox
import datetime
import mylib_file
import mylib_tilt
import easyform_edit

# 入力パラメータの定義
def parse_args():
    # タイトル・バージョン情報
    print(GREEN)
    print('--- {} ---'.format(title))
    print(' OpenCV version {} '.format(cv2.__version__))
    print(NOCOLOR)

    parser = argparse.ArgumentParser()
    parser.add_argument('--log', metavar = 'LOG',
            default = 'n',
            help = 'Log flag.(y/n) Default value is \'n\'')
    parser.add_argument('-o', '--out', metavar = 'CSV_OUT',
            default = DEF_CSV_PATH,
            help = 'CSV file path. Default value is '+DEF_CSV_PATH)
    parser.add_argument('--mlt', metavar = 'MULTI',
            default = 'y',
            help = 'Multi flag.(y/n) Default value is \'y\'')
    parser.add_argument('--tilt', metavar = 'TILT',
            default = 'n',
            help = 'Tilt Check flag.(y/n) Default value is \'n\'')
    return parser

# 入力パラメータの表示
def display_info(logflg, outpath, mltflg, tltflg):
    print(YELLOW + title + ': Starting application...' + NOCOLOR)
    print('   - ' + YELLOW + 'Log flag     : ' + NOCOLOR, logflg)
    print('   - ' + YELLOW + 'Processed out: ' + NOCOLOR, outpath)
    print('   - ' + YELLOW + 'Multi flag   : ' + NOCOLOR, mltflg)
    print('   - ' + YELLOW + 'Tilt chk flag: ' + NOCOLOR, tltflg)
    print('')

# 基本情報を読む
def read_base_info1(myyaml, section):
    path = myyaml.load_section_key(section, '.FileName')
    return path

def read_base_info2(myyaml, section):
    base_cord = myyaml.load_section_key(section, '.BasicCoordinates')
    return base_cord

# 配置情報を読む
def read_area_info(myyaml, section, sub_sec) :
    locat = myyaml.load_section_sub_key(section, sub_sec, 'Location')
    prepros = myyaml.load_section_sub_key(section, sub_sec, 'PreProcess')
    text = myyaml.load_section_sub_key(section, sub_sec, 'Text')
    offset = myyaml.load_section_sub_key(section, sub_sec, 'Offset')
    return locat, prepros, text, offset

# 登録セクション名(ファイル名) を読む out: キー・リスト
def read_section_name(myyaml):
    templ_lst = myyaml.load_section('Template')
    return templ_lst

# OpenCV イメージファイル読む (日本語ファイル名対応)
def imread(filename):
    if platform.system()=='Windows':
        tmp_dir = os.getcwd()
        # 1. 対象ファイルがあるディレクトリに移動
        if len(filename.split("/")) > 1:
            file_dir = "/".join(filename.split("/")[:-1])
            os.chdir(file_dir)
        # 2. 対象ファイルの名前を変更
        tmp_name = "tmp_name"
        os.rename(filename.split("/")[-1], tmp_name)
        # 3. 対象ファイルを読み取る
        img = cv2.imread(tmp_name)
        # 4. 対象ファイルの名前を戻す
        os.rename(tmp_name, filename.split("/")[-1])
        # カレントディレクトリをもとに戻す
        os.chdir(tmp_dir)
    else:
        img = cv2.imread(filename)

    return img

# 帳票 OCR・メイン処理
def image_ocr_process(filename, outpath, logflag=False, regflag = True):
    global section_name, key_table, input_name, key_disp

    # -- 読み取りテキストの変換
    def text_convert(data):
        my_text_convert = mylib_text.TextConvert(False)
        s = [''] * table_n
        s[0] = my_text_convert.replace_key_strings(data[0])     # 請求書
        s[1] = my_text_convert.get_str2date(data[1])            # 日  付
        s[2] = my_text_convert.replace_company_strings(data[2]) # 会社名
        s[3] = my_text_convert.replace_Title_strings(data[3])   # 案件名
        s[4] = my_text_convert.get_str2mony(data[4])            # 税抜金額
        s[5] = my_text_convert.get_str2mony(data[5])            # 税込金額
        return s

    # -- CSV 出力
    def csv_output(row_data):
        global section_name, key_table, input_name, key_disp

        encoding = 'utf_8_sig'
        key = 'ファイル名'
        form ='適用フォーム'
        my_csv_treatment = mylib_csv.CSVtreatment(outpath, encoding, False)
        s = my_csv_treatment.read_csv()
        if len(s) == 0:
            data = copy.copy(key_disp)
            data.insert(0, key)
            data.append(form)
            my_csv_treatment.write_csv(data)

        isSaved = my_csv_treatment.check_csv(key, input_name)
        if not isSaved:
            data = copy.copy(row_data)
            data.insert(0, input_name)
            my_csv_treatment.append_csv(data)

        if logflag:
            print(' Output Data: ', row_data)

        return not isSaved

    # -- OCR 処理
    def ocr_process():
        global section_name, key_table, input_name, key_disp

        my_text_convert = mylib_text.TextConvert(False)
        sr = ' Check → '
        result = False
        form_ofsx = 0
        form_ofsy = 0
        for n in range(table_n):
            # 配置情報の読み出し
            ocr_rect, pros, ocr_text, offset = read_area_info(myyaml_map, section_name, key_table[n])
            
            if offset == None:
                ofsx = 0
                ofsy = 0
            else:
                ofsx = offset[0]
                ofsy = offset[1]
            if pros == None:
                pros = 0
            prepros = int(pros)
            preprocess_mode[3], mod = divmod(prepros, 8)
            preprocess_mode[2], mod = divmod(mod, 4)
            preprocess_mode[1], preprocess_mode[0] = divmod(mod, 2)

            if ocr_text == None:
                break
            text_tmpl[n] = ocr_text

            if ocr_rect == None:
                break
            else:
                # オフセット補正
                ocr_rect[0] = ocr_rect[0] + form_ofsx
                ocr_rect[1] = ocr_rect[1] + form_ofsy
                ocr_rect[2] = ocr_rect[2] + form_ofsx
                ocr_rect[3] = ocr_rect[3] + form_ofsy

                # オリジナル座標 → 表示座標
                x0, y0 = imgfr.get_org2res_xy(round(ocr_rect[0] / base_scale_w), round(ocr_rect[1] / base_scale_h))
                x1, y1 = imgfr.get_org2res_xy(round(ocr_rect[2] / base_scale_w), round(ocr_rect[3] / base_scale_h))
                roi.x = x0
                roi.y = y0
                roi.width = x1 - x0
                roi.height = y1 - y0

            if roi.area() > 50:
                # 得られた表示座標から元画像の位置を計算して画像を切り出す
                #x0, y0 = imgfr.get_res2org_xy(roi.x, roi.y)
                #x1, y1 = imgfr.get_res2org_xy(roi.x + roi.width, roi.y + roi.height)

                # フォームの座標を元座標の座標系に変換して画像を切り出す
                x0 = round(ocr_rect[0] / base_scale_w)
                y0 = round(ocr_rect[1] / base_scale_h)
                x1 = round(ocr_rect[2] / base_scale_w)
                y1 = round(ocr_rect[3] / base_scale_h)
                lenaRoi = lena_frame_org[y0 : y1, x0 : x1]
                ocr_rect = [x0, y0, x1, y1]

                text = ''
                if lenaRoi.shape[0] > 0 and lenaRoi.shape[1] > 0:
                    # 前処理
                    if preprocess_mode[3] != 0:
                        lenaRoi = imgpros.image_processing_execution(lenaRoi, 1)
                    if preprocess_mode[2] != 0:
                        lenaRoi = imgpros.image_processing_execution(lenaRoi, 4)
                    if preprocess_mode[1] != 0:
                        lenaRoi = imgpros.image_processing_execution(lenaRoi, 2)
                    if preprocess_mode[0] != 0:
                        lenaRoi = imgpros.image_processing_execution(lenaRoi, 5)

                    # 切り出した領域を OCR
                    text, x0, y0, x1, y1 = image2string(lenaRoi, n)

                # テキストの保存
                text_save[n] = str(text)
                if len(text) > 0:
                    if n == 0:                              # 請求書
                        # フォーム位置ずれ計算
                        form_ofsx = 0 if ofsx == 0 and ofsy == 0 else round(x0 * base_scale_w) - ofsx 
                        form_ofsy = 0 if ofsx == 0 and ofsy == 0 else round(y0 * base_scale_h) - ofsy

                        tmpl = my_text_convert.replace_key_strings(text_tmpl[n])
                        text = my_text_convert.replace_key_strings(text_save[n])

                        if logflag:
                            print(YELLOW + ' * tmpl text:', text_tmpl[n], '→', tmpl)
                            print(' * ocr  text:', text_save[n], '→', text,  NOCOLOR)


                        if text == tmpl:
                            sr = sr + GREEN + ' {}:Yes'.format(n)
                        else:
                            sr = sr + YELLOW + ' {}:No'.format(n)
                            break
                    if n == 1:                              # 日  付
                        text = my_text_convert.get_str2date(text_save[n])
                        if logflag:
                            print(YELLOW + ' * ocr  text:', text_save[n], '→', text,  NOCOLOR)
                        if len(text) > 0:
                            sr = sr + GREEN + ' {}:Yes'.format(n)
                        else:
                            sr = sr + YELLOW + ' {}:No'.format(n)
                            break
                    if n == 2:                              # 会社名
                        text = my_text_convert.replace_company_strings(text_save[n])
                        if logflag:
                            print(YELLOW + ' * ocr  text:', text_save[n], '→', text,  NOCOLOR)
                        sr = sr + GREEN + ' {}:Yes'.format(n)
                    if n == 3:                              # 案件名
                        text = my_text_convert.replace_Title_strings(text_save[n])
                        if logflag:
                            print(YELLOW + ' * ocr  text:', text_save[n], '→', text,  NOCOLOR)
                        sr = sr + GREEN + ' {}:Yes'.format(n)
                    if n == 4:                              # 税抜金額
                        text = my_text_convert.get_str2mony(text_save[n])
                        if logflag:
                            print(YELLOW + ' * ocr  text:', text_save[n], '→', text,  NOCOLOR)
                        if len(text) > 1:                   #  2桁以上
                            sr = sr + GREEN + ' {}:Yes'.format(n)
                        else:
                            sr = sr + YELLOW + ' {}:No'.format(n)
                            break
                    if n == 5:                              # 税込金額
                        text0 = my_text_convert.get_str2mony(text_save[n - 1])
                        text = my_text_convert.get_str2mony(text_save[n])
                        if logflag:
                            print(YELLOW + ' * ocr  text:', text_save[n], '→', text,  NOCOLOR)
                            print(YELLOW + ' * total mony:', text_save[n - 1], '→', text0)
                            print(' * b4tax:', text_save[n], '→', text,  NOCOLOR)
                        if len(text) > 1:                   #  2桁以上
                            mony0 = int(text0)
                            mony1 = int(text)
                            if mony1 > mony0:               # 税込金額のほうが大きい 
                                sr = sr + GREEN + ' {}:Yes all match'.format(n)
                                result = True
                            else:
                                sr = sr + YELLOW + ' {}:No*'.format(n)
                                break
                        else:
                            sr = sr + YELLOW + ' {}:No'.format(n)
                            break

                    if logflag:
                        if n == 0:
                            print(' text ofset : ', x0, y0)
                            print(' form value : ', ofsx, ofsy)
                            print(' base scale : {:.3f} x {:.3f}'.format(base_scale_w, base_scale_h))
                            print(' form Ofset : ', form_ofsx, form_ofsy)
                        msg = 'preprocess: {}  area: ({}, {}) - ({}, {})'.format(preprocess_mode, ocr_rect[0], ocr_rect[1], ocr_rect[2], ocr_rect[3])
                        print(' ', msg)
                        print(' ', text)
                else:
                    break

            else:
                break

            if result:
                break

        if logflag:
            print(sr + NOCOLOR)

        return result

    # -- 画像データから OCR テキスト
    def image2string(img, mode):
        text = ''
        x0 = 0
        y0 = 0
        x1 = 0
        y1 = 0
        if mode == 4 or mode == 5:
            reader = easyocr.Reader(['en'], gpu=False)
        else :
            reader = easyocr.Reader(['ja', 'en'], gpu=False)
        result = reader.readtext(img)
        for obj in result:
            x0 = int(obj[0][0][0])                # 結果が浮動小数点になる場合があるので
            y0 = int(obj[0][0][1])
            x1 = int(obj[0][2][0])
            y1 = int(obj[0][2][1])
            text = obj[1]
            confidence = obj[2]

        return text, x0, y0, x1, y1

    # -- テンプレートの一致を調べる  in; テキストリスト form_tx, read_tx 
    def form_check(form_tx, read_tx):
        cnv_form_tx = text_convert(form_tx)
        cnv_read_tx = text_convert(read_tx)
        result = False
        if cnv_form_tx[0] == cnv_read_tx[0]:            # 請求書項目が一致
            if len(cnv_read_tx[1]) > 0:                 # 日付が正しい
                if len(cnv_read_tx[4]) > 1:             # 税抜金額が２桁以上
                    if len(cnv_read_tx[5]) > 1:         # 税込金額が２桁以上
                        mony0 = int(cnv_read_tx[4])
                        mony1 = int(cnv_read_tx[5])
                        if mony1 > mony0:               # 税込金額のほうが大きい 
                            result = True

        return result
    # --

    if logflag:
        print('\n file: <{}>'.format(filename))

    # OpenCV でイメージを読む
    lena_frame_org = imread(filename)
    key = -1
    if lena_frame_org is None:
        print(RED + "\nUnable to read the input." + NOCOLOR)
        return key

    input_name = os.path.basename(filename)
    output_name = os.path.basename(outpath)
    WINDOW_NAME = title
    print(' << {} >> '.format(input_name))

    preprocess_mode = [0, 0, 0, 0]
    set_mode = 0
    wlock1 = 0

    # 配置情報読み出し・設定処理
    conf_file = CONFIG_FILE
    map_file = MAPING_FILE
    myyaml_cnf = mylib_yaml.YamlProcess(conf_file, False)
    myyaml_map = mylib_yaml.YamlProcess(map_file, False)
    key_table = myyaml_cnf.load_section('KeyTable')
    key_disp = myyaml_cnf.load_section('KeyDisp')
    
    load_mode = [0] * len(key_table)
    status_h = 40                               # ステータスラインの高さ
    
    # 日本語フォント指定
    if platform.system()=='Windows':
       fontPIL = 'meiryo.ttc'                   # メイリオ
    else:
       fontPIL = 'NotoSansCJK-Bold.ttc'         # ゴシック体

    # ディスプレイ解像度を得る (Ubuntu の場合 height - 64)
    monitor_height, monitor_width = mylib_screen.get_display_size(logflag)
    maxsize = monitor_height - 64 - 100

    # 画像の前処理
    imgpros = mylib_preprocess.ImagePreprocess(False)    # 初期化

    # mylib_frame ライブラリ
    imgfr = mylib_frame.ImageFrame(lena_frame_org)       # 初期化
    imgfr.set_screen_size(monitor_width, monitor_height)

    lena_frame = imgfr.frame_resize(maxsize)
    lena_frame_h, lena_frame_w = lena_frame.shape[:2]

    # 画面ステータス領域 (画面下部 status_h pixel)の確保
    frame = np.zeros((lena_frame_h + status_h, lena_frame_w, 3), np.uint8)
    frame[:,:,:] = 200

    btn_x = 90
    btn_y = lena_frame_h + 4
    btn_w = 70
    btn_h = 32
    
    popup_frame = np.zeros((120, 500, 3), np.uint8)
    anchor = cvui.Point()
    roi = cvui.Rect(0, 0, 0, 0)
    frame_h, frame_w = frame.shape[:2]
    csvf = False
    out_mode = 0
    out_count = 0

    org_h, org_w = imgfr.get_original_size()
    scale_h, scale_w = imgfr.get_scale()

    # 登録テンプレートを検索
    template_tbl = read_section_name(myyaml_cnf)
    ret = False
    if template_tbl != None:
        if input_name in template_tbl:     # 登録があればテーブルの最後に
            template_tbl.remove(input_name)
            template_tbl.append(input_name)
        template_tbl.reverse()              # 最新の登録から検索

        for section_name in template_tbl:
            base_cord = read_base_info2(myyaml_map, section_name)
            if base_cord != None:
                base_scale_w = base_cord[0] / org_w
                base_scale_h = base_cord[1] / org_h
                table_n = len(key_table)

                if logflag:
                    print('\n Check Form : < {} >'.format(section_name))
                    print(' original h x w : {:=5} x {:=5}'.format(org_h, org_w))
                    print(' display  h x w : {:=5} x {:=5}'.format(lena_frame_h, lena_frame_w))
                    print(' scale    h x w : {:.3f} x {:.3f}'.format(scale_h, scale_w))
                    print(' tp-size  h x w : {:=5} x {:=5}'.format(base_cord[1], base_cord[0]))
                    print(' tp-scale h x w : {:.3f} x {:.3f}'.format(base_scale_h, base_scale_w))
                    print(' -----------')

                # OCR 項目設定データの保存場所
                text_tmpl = [''] * table_n
                text_save = [''] * table_n
                text_cnvt = [''] * table_n
                csv_savef = False

                ret = ocr_process()

            if logflag:
                if ret:
                    print('\n Template Form: ', section_name)
                else:
                    print(YELLOW + ' ** Form mismatch !! **' + NOCOLOR +' < {} >'.format(section_name))
            if ret:
                break

        template_tbl.reverse()              # 順序をもとに戻す
    my_file = mylib_file.FileTreatment(False)
    key = ord('q')
    if ret:                                 # フォームが一致
        ss = '*' if input_name == section_name else ''
        print(GREEN + ' matching form: {} {}'.format(section_name, ss) + NOCOLOR)
        text_cnvt = text_convert(text_save)
        text_cnvt.append(section_name)

        if logflag:
            print(' Outout file: ', output_name)

        ret = csv_output(text_cnvt)
        if not ret:
            print(GREEN + ' ** Already recorded !! **' + NOCOLOR)

        # -err フォルダ内のファイルの場合もとに戻す
        if my_file.ckeck_erfolder(filename):
            my_file.move_file(filename)

        # 一致したフォームをテーブルの最後に置き直す
        template_tbl.remove(section_name)
        template_tbl.append(section_name)
        myyaml_cnf.write_append('Template', template_tbl)

        key = ord('n')
    else:                                   # 一致するフォームがない
        print(RED + ' -- No matching form ! -- << {} >> '.format(input_name) + NOCOLOR)
        is_errfol = my_file.ckeck_erfolder(filename)
        ret = False
        if is_errfol or not regflag:
            msg = '< {} >\n\n登録されていません. 編集しますか？'.format(input_name)
            ret = messagebox.askyesno('確認', msg)
        if ret:
            key = easyform_edit.image_annotation(filename, logflag, False)
        else:
            # -err フォルダに移動する
            if not is_errfol:
                my_file.move_file(filename)

            key = ord('n')

    return key


# ** main関数 **
def main():
    loop_flg = True

    # Tk オブジェクトのインスタンス作成
    root = tkinter.Tk()
 
    # Tk のウィンドウを非表示にする
    root.withdraw()
    
    # 入力パラメータの処理
    ARGS = parse_args().parse_args()
    logflg = ARGS.log
    logflag = True if logflg == 'y' else False 
    outpath = ARGS.out
    mltflg = ARGS.mlt
    mltflag = True if mltflg == 'y' else False 
    tltflg = ARGS.tilt
    tltflag = True if tltflg == 'y' else False 

    # 情報表示
    display_info(logflg, outpath, mltflg, tltflg)

    # 登録テンプレートを検索
    myyaml_cnf = mylib_yaml.YamlProcess(CONFIG_FILE, False)
    template_tbl = read_section_name(myyaml_cnf)
    regflag = False if template_tbl == None else True

    if mltflag:
        # 指定フォルダ内の画像ファイルを一括処理する
        dirname = filedialog.askdirectory(initialdir = './', title = '対象ディレクトリを選択', mustexist = True)
        if len(dirname) > 0:
            if tltflag:
                t = datetime.datetime.now()
                print(' >> tilt check >>', t.time().strftime('%X'))
                mylib_tilt.tilt_image_fol(dirname, 100, 200, 20, 'non', logflag, True)

            t = datetime.datetime.now()
            print('\n >> OCR process >>', t.time().strftime('%X'))
            my_file = mylib_file.FileTreatment(False)
            img_ext = {'.bmp', '.png', '.jpeg', '.jpg', '.tif'}
            imglist = my_file.get_file_list_sel(dirname + '/*', img_ext)

            for filename in imglist:
                retryf = True
                while (retryf):
                    ret = image_ocr_process(filename, outpath, logflag, regflag)
                    if ret == 27 or ret == 113 or ret == -1:
                        retryf = False
                        break
                    elif ret == ord('s'):
                        print(' ** Run again !! **')
                        retryf = True
                    else:                    
                        retryf = False

                if ret == 27 or ret == 113:
                    break
    else:
        # 指定画像ファイルの処理
        filename = filedialog.askopenfilename(title = '画像ファイルを開く', filetypes = [("Image file", ".bmp .png .jpeg .jpg .tif"), ("Bitmap", ".bmp"), ("PNG", ".png"), ("JPEG", ".jpeg"), ("JPG", ".jpg")], initialdir = './')
        if len(filename) > 0:
            if tltflag:
                print(' >> tilt check >>')
                mylib_tilt.tilt_image(filename, 100, 200, 20, 'non', logflag, True)

            print('\n >> OCR process >>')
            retryf = True
            while (retryf):
                ret = image_ocr_process(filename, outpath, logflag, regflag)
                if ret == 27 or ret == 113 or ret == -1:
                    retryf = False
                    break
                elif ret == ord('s'):
                    print(' ** Run again !! **')
                    retryf = True
                else:                    
                    retryf = False
    t = datetime.datetime.now()
    print('\n Finished. ', t.time().strftime('%X'))

# main関数エントリーポイント(実行開始)
if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print(RED + ' Keyboard Interrupt !!' + NOCOLOR)

