# -*- coding: utf-8 -*-
##------------------------------------------
##   My Library CSV file treatment  Ver 0.04
##
##               2022.01.18 Masahiro Izutsu
##------------------------------------------
## mylib_csv.py
##      Ver 0.04    リストファイル読み出し対応       2022.04.12

import csv

class CSVtreatment:
    logf = False
    encoding = ''
    path = ''
    
    # 初期化
    #   flg:        ログ出力フラグ
    def __init__(self, path, encoding = 'utf_8_sig', flg = False):
        self.path = path 
        self.encoding = encoding
        self.logf = flg

    # CSV ファイル書き込み  in:[] 文字列リスト
    def write_csv(self, row_data):
        try:
            with open(self.path, 'w', encoding = self.encoding) as f:
                writer = csv.writer(f, lineterminator='\n')
                writer.writerow(row_data)
        except FileNotFoundError as e:
            if self.logf:
                print(e)
        except csv.Error as e:
            if self.logf:
                print(e)

    # CSV ファイル追加書き込み  in:[] 文字列リスト
    def append_csv(self, row_data):
        try:
            with open(self.path, 'a', encoding = self.encoding) as f:
                writer = csv.writer(f, lineterminator='\n')
                writer.writerow(row_data)
        except FileNotFoundError as e:
            if self.logf:
                print(e)
        except csv.Error as e:
            if self.logf:
                print(e)

    # CSV ファイル読み出し  out: 文字列
    def read_csv(self):
        try:
            with open(self.path, 'r', encoding = self.encoding) as f:
                string = f.read()
        except FileNotFoundError as e:
            string = ''
            if self.logf:
                print(e)
        except csv.Error as e:
            string = ''
            if self.logf:
                print(e)
            
        return string

    # CSV ファイルチェック (key = value があるかを調べる)
    def check_csv(self, key, value):
        try:
            with open(self.path, 'r', encoding = self.encoding) as f:
                reader = csv.DictReader(f)
                s = [row for row in reader]
                s0 = next((x for x in s if x[key] == value), None)
                if self.logf:
                    print('[mylib_csv:] ', s0)
        except FileNotFoundError as e:
            s0 = None
            if self.logf:
                print(e)
        except csv.Error as e:
            s0 = None
            if self.logf:
                print(e)
            
        return s0 != None

    # CSV ファイル 文字列リスト2読み出し  out: 文字列リスト1, 文字列リスト2
    def read_list2_csv(self):
        befor = []
        after = []
        try:
            with open(self.path, 'r', encoding = self.encoding) as f:
                reader = csv.reader(f)
                for row in reader:
                    befor.append(str(row[0]))
                    after.append(str(row[1]))
        except FileNotFoundError as e:
            if self.logf:
                print(e)
        except csv.Error as e:
            if self.logf:
                print(e)
            
        return befor, after

##---------------
def main():
    path = 'sample_csv_utf8sig.csv'
    encoding = 'utf_8_sig'
    my_csv_treatment = CSVtreatment(path, encoding, True)

    row_data = ['INDEX', '姓', '名']
    my_csv_treatment.write_csv(row_data)
    row_data = [1, '井筒', '政弘']
    my_csv_treatment.append_csv(row_data)
    row_data = [2, '新田', '孝']
    my_csv_treatment.append_csv(row_data)

    string = my_csv_treatment.read_csv()
    print(type(string))
    print(string)

    path = 'tryocr.csv'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, True)

    string = my_csv_treatment1.read_csv()
    print(type(string))
    print(string)

    key = 'ファイル名'
    value = 'sample0.png'
    print (my_csv_treatment1.check_csv(key, value))

    path = 'dic_key.txt'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, True)

    befor, after = my_csv_treatment1.read_list2_csv()
    print(type(befor), type(after))
    print(befor, '\n', after)

    path = 'dic_title.txt'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, True)

    befor, after = my_csv_treatment1.read_list2_csv()
    print(type(befor), type(after))
    print(befor, '\n', after)

    path = 'dic_company.txt'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, True)

    befor, after = my_csv_treatment1.read_list2_csv()
    print(type(befor), type(after))
    print(befor, '\n', after)

    path = 'dic_date.txt'
    encoding = 'utf_8_sig'
    my_csv_treatment1 = CSVtreatment(path, encoding, True)

    befor, after = my_csv_treatment1.read_list2_csv()
    print(type(befor), type(after))
    print(befor, '\n', after)


if __name__ == "__main__":
    main()

