# -*- coding: utf-8 -*-
##------------------------------------------
##   My Library Text Convert    Ver 0.04
##
##               2022.01.17 Masahiro Izutsu
##------------------------------------------
## mylib_text.py
##      Ver 0.01    「令和」対応            2022.02.19
##      Ver 0.04    辞書ファイル対応        2022.04.12
##                      変換前文字列1,変換後文字列1
##                      変換前文字列2,変換後文字列2
##                          :

import re
import datetime
import csv
import mylib_csv

MODULE_NAME = 'TextConvert'
ENCORDING = 'utf_8_sig'
DIC_KEY_FILE = 'dic_key.txt'
DIC_TITLE_FILE = 'dic_title.txt'
DIC_COMPANY_FILE = 'dic_company.txt'
DIC_DATE_FILE = 'dic_date.txt'

class TextConvert:
    logf = False
    
    # 初期化
    #   flg:        ログ出力フラグ
    def __init__(self, flg = False):
        self.logf = flg

    # 文字列から数字を取り出す  out: 数字のリスト  error=空のリスト
    def get_str2num(self, text):
        result = re.findall(r"\d+", text)
        return result

    # 年月日の数値の妥当性を調べる  out: 文字列  error=''
    def checkDate(self, year, month, day):
        try:
            str = datetime.date(int(year), int(month), int(day))
            string = str.strftime('%Y/%m/%d')
        except ValueError:
            string = ''
            if self.logf:
                print('[{}:checkDate()] Invalid string error !!'.format(MODULE_NAME))
        return string

    # 文字列が年月日かどうかを調べる(「令和」対応)  out: 文字列  error=''
    def get_str2date(self, text):
        string = ''
        text = re.sub('[,.、]', '', text)
        text = self.replace_day_strings(text)
        result = self.get_str2num(text)
        if len(result) == 3:
            if int(result[0]) < 2000:
                text = self.remove_space(text)
                result = self.get_str2num(text)
        if len(result) != 3:
            text = self.remove_space(text)
            result = self.get_str2num(text)
        if len(result) == 3:
            if text.find('令和') == 0:
                reiwa = int(result[0]) + 2018
                result[0] = str(reiwa)
            string = self.checkDate(result[0], result[1], result[2])

        elif len(result) == 2:
            if text.find('令和元年') == 0:
                result.insert(0, 2019)
                string = self.checkDate(result[0], result[1], result[2])

        else:
            if self.logf:
                print('[{}:get_str2date()] Invalid string error !!'.format(MODULE_NAME))

        return string

    # 文字列から金額を取り出す  out: 文字列  error=''
    def get_str2mony(self, text):
        text = re.sub('[CcoO]', '0', text)
        text0 = self.remove_space(text)
        money = ''
        if len(text0) > 0:
            txt = re.sub('[,.、]', '', text0)
            if len(txt) > 0:
                result = self.get_str2num(txt)
                if len(result) == 1:
                    money = str(result[0])
        return money

    # 文字列の空白を取る  out: 文字列  error=''
    def remove_space(self, text):
        string = re.sub(r"[\u3000 \t]", "", text)
        return string


    def remove_crlf(self, text):
        text = text.replace('\n', '')
        return text

    # よくある誤読をヒューリスティックに訂正(キー文字列)
    def replace_key_strings(self, text):
        text = self.remove_space(text)
        text = re.sub('[。_.-<-”]', '', text)

        my_csv = mylib_csv.CSVtreatment(DIC_KEY_FILE, ENCORDING, self.logf)
        befor, after = my_csv.read_list2_csv()
        n = len(befor)
        if n > 0:
            for i in range(n):
                text = text.replace(befor[i], after[i])
                text = text.replace(u'請請求書', u'請求書')
        return text

    # よくある誤読をヒューリスティックに訂正(タイトル)
    def replace_Title_strings(self, text):
        text = self.remove_crlf(text)
        text = self.remove_space(text)
        text = re.sub('[。、_|]', '', text)

        my_csv = mylib_csv.CSVtreatment(DIC_TITLE_FILE, ENCORDING, self.logf)
        befor, after = my_csv.read_list2_csv()
        n = len(befor)
        if n > 0:
            for i in range(n):
                text = text.replace(befor[i], after[i])
                text = text.replace(u'ソソフトウェア', u'ソフトウェア')
                text = text.replace(u'ソフブソフトウェア', u'ソフトウェア')
                text = text.replace(u'ソフソフトウェア', u'ソフトウェア')
        return text

    # よくある誤読をヒューリスティックに訂正(会社名)
    def replace_company_strings(self, text):
        text = self.remove_crlf(text)
        text = self.remove_space(text)
        text = re.sub("[$@&=]","", text)
        text = re.sub('[。、_,|0]', '', text)

        my_csv = mylib_csv.CSVtreatment(DIC_COMPANY_FILE, ENCORDING, self.logf)
        befor, after = my_csv.read_list2_csv()
        n = len(befor)
        if n > 0:
            for i in range(n):
                text = text.replace(befor[i], after[i])
        return text

    # よくある誤読をヒューリスティックに訂正(日付)
    def replace_day_strings(self, text):
        my_csv = mylib_csv.CSVtreatment(DIC_DATE_FILE, ENCORDING, self.logf)
        befor, after = my_csv.read_list2_csv()
        n = len(befor)
        if n > 0:
            for i in range(n):
                text = text.replace(befor[i], after[i])
        return text

##---------------
def main():
    my_text_convert = TextConvert(True)

    # 日付文字列
    s = '信和 元 年 8 月 31 日'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '2020 年 06 月 80 日'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '2022年1月17日'
    print(' get_str2num():', s, '→ ', my_text_convert.get_str2num(s))
    s = '年月日'
    print(' get_str2num():', s, '→ ', my_text_convert.get_str2num(s))
    year = 2022
    month = 2
    day = 31
    print(' checkDate():', year, month,day, '→ ',my_text_convert.checkDate(year, month,day))
    month = 1
    print(' checkDate():', year, month,day, '→ ',my_text_convert.checkDate(year, month,day))
    s = '2022年1月17日'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '2022年2月31日'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '2月31日'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '20,22年1月31日'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '令和元年1月2日'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '令和4年2月14日'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '令 和 元 年 8 月 31 日'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '2021 12 31'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '令 和 3 年 12 月 31 日'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    #----------------

    s = ' \8,500'
    print(' get_str2mony():', s, '→ ', my_text_convert.get_str2mony(s))
    s = '590, 000'
    print(' get_str2mony():', s, '→ ', my_text_convert.get_str2mony(s))
    s = '\\637, 200'
    print(' get_str2mony():', s, '→ ', my_text_convert.get_str2mony(s))
    s = '615、600'
    print(' get_str2mony():', s, '→ ', my_text_convert.get_str2mony(s))
    s = '1, 259, 020'
    print(' get_str2mony():', s, '→ ', my_text_convert.get_str2mony(s))
    s = '526.845'
    print(' get_str2mony():', s, '→ ', my_text_convert.get_str2mony(s))
    s = '2,070, 490'
    print(' get_str2mony():', s, '→ ', my_text_convert.get_str2mony(s))
    s = '2, 277, 939'
    print(' get_str2mony():', s, '→ ', my_text_convert.get_str2mony(s))
    #----------------

    s = '請 求 書'
    print(' remove_space():', s, '→ ', my_text_convert.remove_space(s))
    s = '講 求 書'
    print(' replace_key_strings():', s, '→ ', my_text_convert.replace_key_strings(s))
    s = '請 。 求 書'
    print(' replace_key_strings():', s, '→ ', my_text_convert.replace_key_strings(s))
    s = '請 _ 求 _ 書'
    print(' replace_key_strings():', s, '→ ', my_text_convert.replace_key_strings(s))
    s = '著 求 書'
    print(' replace_key_strings():', s, '→ ', my_text_convert.replace_key_strings(s))
    s = 'ジブフトウェア'
    print(' replace_Title_strings():', s, '→ ', my_text_convert.replace_Title_strings(s))
    s = 'アプクシアプアソフトデザイン'
    print(' replace_company_strings():', s, '→ ', my_text_convert.replace_company_strings(s))
    s = 'PPFパパートナーズ'
    print(' replace_company_strings():', s, '→ ', my_text_convert.replace_company_strings(s))
    s = '株式会会社ントラル情報サービス'
    print(' replace_company_strings():', s, '→ ', my_text_convert.replace_company_strings(s))
    s = 'フトウェア開発'
    print(' replace_Title_strings():', s, '→ ', my_text_convert.replace_Title_strings(s))
    s = 'ソソフトウェア開発'
    print(' replace_Title_strings():', s, '→ ', my_text_convert.replace_Title_strings(s))
    s = 'インター・ラボ株式会社='
    print(' replace_company_strings():', s, '→ ', my_text_convert.replace_company_strings(s))
    s = 'ソフブフトウェア'
    print(' replace_Title_strings():', s, '→ ', my_text_convert.replace_Title_strings(s))
    s = '2020 年 06 月 90 日'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '2019 の 31'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '2,520, 00C'
    print(' get_str2mony():', s, '→ ', my_text_convert.get_str2mony(s))
    s = '峡 <- 求 書'
    print(' replace_key_strings():', s, '→ ', my_text_convert.replace_key_strings(s))
    s = '針 和 3 年 12 月 31 日'
    print(' get_str2date():', s, '→ ', my_text_convert.get_str2date(s))
    s = '2 トウェア 開発 業務'
    print(' replace_Title_strings():', s, '→ ', my_text_convert.replace_Title_strings(s))
    s = 'トウェア開発業務'
    print(' replace_Title_strings():', s, '→ ', my_text_convert.replace_Title_strings(s))

if __name__ == "__main__":
    main()

